@@ -41,11 +41,12 @@ struct ares__htable {
   unsigned int               seed;
   unsigned int               size;
   size_t                     num_keys;
+  size_t                     num_collisions;
   /* NOTE: if we converted buckets into ares__slist_t we could guarantee on
    *       hash collisions we would have O(log n) worst case insert and search
    *       performance.  (We'd also need to make key_eq into a key_cmp to
    *       support sort).  That said, risk with a random hash seed is near zero,
-   *       and ares__slist_t is heavier weight so I think using ares__llist_t is
+   *       and ares__slist_t is heavier weight, so I think using ares__llist_t
    *       is an overall win. */
   ares__llist_t            **buckets;
 };
@@ -161,6 +162,9 @@ static ares_bool_t ares__htable_expand(ares__htable_t *htable)
   ares__llist_t **buckets  = NULL;
   unsigned int    old_size = htable->size;
   size_t          i;
+  ares__llist_t **prealloc_llist     = NULL;
+  size_t          prealloc_llist_len = 0;
+  ares_bool_t     rv                 = ARES_FALSE;
 
   /* Not a failure, just won't expand */
   if (old_size == ARES__HTABLE_MAX_BUCKETS) {
@@ -169,28 +173,47 @@ static ares_bool_t ares__htable_expand(ares__htable_t *htable)
 
   htable->size <<= 1;
 
-  /* We must do this in 2 passes as we want it to be non-destructive in case
-   * there is a memory allocation failure.  So we will actually use more
-   * memory doing it this way, but at least we might be able to gracefully
-   * recover */
+  /* We must pre-allocate all memory we'll need before moving entries to the
+   * new hash array.  Otherwise if there's a memory allocation failure in the
+   * middle, we wouldn't be able to recover. */
   buckets = ares_malloc_zero(sizeof(*buckets) * htable->size);
   if (buckets == NULL) {
-    goto fail;
+    goto done;
   }
 
+  /* The maximum number of new llists we'll need is the number of collisions
+   * that were recorded */
+  prealloc_llist_len = htable->num_collisions;
+  if (prealloc_llist_len) {
+    prealloc_llist =
+      ares_malloc_zero(sizeof(*prealloc_llist) * prealloc_llist_len);
+    if (prealloc_llist == NULL) {
+      goto done;
+    }
+  }
+  for (i = 0; i < prealloc_llist_len; i++) {
+    prealloc_llist[i] = ares__llist_create(htable->bucket_free);
+    if (prealloc_llist[i] == NULL) {
+      goto done;
+    }
+  }
+
+  /* Iterate across all buckets and move the entries to the new buckets */
+  htable->num_collisions = 0;
   for (i = 0; i < old_size; i++) {
     ares__llist_node_t *node;
 
     /* Nothing in this bucket */
-    if (htable->buckets[i] == NULL)
+    if (htable->buckets[i] == NULL) {
       continue;
+    }
 
-    /* Fast past optimization (most likely case), there is likely only a single
+    /* Fast path optimization (most likely case), there is likely only a single
      * entry in both the source and destination, check for this to confirm and
      * if so, just move the bucket over */
     if (ares__llist_len(htable->buckets[i]) == 1) {
-      void  *val = ares__llist_first_val(htable->buckets[i]);
-      size_t idx = HASH_IDX(htable, htable->bucket_key(val));
+      const void *val = ares__llist_first_val(htable->buckets[i]);
+      size_t      idx = HASH_IDX(htable, htable->bucket_key(val));
 
       if (buckets[idx] == NULL) {
         /* Swap! */
@@ -202,8 +225,8 @@ static ares_bool_t ares__htable_expand(ares__htable_t *htable)
 
     /* Slow path, collisions */
     while ((node = ares__llist_node_first(htable->buckets[i])) != NULL) {
-      void  *val = ares__llist_node_val(node);
-      size_t idx = HASH_IDX(htable, htable->bucket_key(val));
+      const void *val = ares__llist_node_val(node);
+      size_t      idx = HASH_IDX(htable, htable->bucket_key(val));
 
       /* Try fast path again as maybe we popped one collision off and the
        * next we can reuse the llist parent */
@@ -214,27 +237,48 @@ static ares_bool_t ares__htable_expand(ares__htable_t *htable)
         break;
       }
 
+      /* Grab one off our preallocated list */
       if (buckets[idx] == NULL) {
-        buckets[idx] = ares__llist_create(htable->bucket_free);
-      }
-      if (buckets[idx] == NULL) {
-        goto fail;
+        /* Silence static analysis, this isn't possible but it doesn't know */
+        if (prealloc_llist == NULL || prealloc_llist_len == 0) {
+          goto done;
+        }
+        buckets[idx] = prealloc_llist[prealloc_llist_len - 1];
+        prealloc_llist_len--;
+      } else {
+        /* Collision occurred since the bucket wasn't empty */
+        htable->num_collisions++;
       }
 
       ares__llist_node_move_parent_first(node, buckets[idx]);
     }
+
+    /* Abandoned bucket, destroy */
+    if (htable->buckets[i] != NULL) {
+      ares__llist_destroy(htable->buckets[i]);
+      htable->buckets[i] = NULL;
+    }
   }
 
-  /* Swap out buckets */
-  ares__htable_buckets_destroy(htable->buckets, old_size, ARES_FALSE);
+  /* We have guaranteed all the buckets have either been moved or destroyed,
+   * so we just call ares_free() on the array and swap out the pointer */
+  ares_free(htable->buckets);
   htable->buckets = buckets;
-  return ARES_TRUE;
+  buckets         = NULL;
+  rv              = ARES_TRUE;
 
-fail:
-  ares__htable_buckets_destroy(buckets, htable->size, ARES_FALSE);
-  htable->size = old_size;
+done:
+  ares_free(buckets);
+  /* destroy any unused preallocated buckets */
+  ares__htable_buckets_destroy(prealloc_llist, (unsigned int)prealloc_llist_len,
+                               ARES_FALSE);
 
-  return ARES_FALSE;
+  /* On failure, we need to restore the htable size */
+  if (rv != ARES_TRUE) {
+    htable->size = old_size;
+  }
+
+  return rv;
 }
 
 ares_bool_t ares__htable_insert(ares__htable_t *htable, void *bucket)
@@ -282,6 +326,11 @@ ares_bool_t ares__htable_insert(ares__htable_t *htable, void *bucket)
     return ARES_FALSE;
   }
 
+  /* Track collisions for rehash stability */
+  if (ares__llist_len(htable->buckets[idx]) > 1) {
+    htable->num_collisions++;
+  }
+
   htable->num_keys++;
 
   return ARES_TRUE;
@@ -316,6 +365,12 @@ ares_bool_t ares__htable_remove(ares__htable_t *htable, const void *key)
   }
 
   htable->num_keys--;
+
+  /* Reduce collisions */
+  if (ares__llist_len(ares__llist_node_parent(node)) > 1) {
+    htable->num_collisions--;
+  }
+
   ares__llist_node_destroy(node);
   return ARES_TRUE;
 }
@@ -344,31 +399,6 @@ unsigned int ares__htable_hash_FNV1a(const unsigned char *key, size_t key_len,
   return hv;
 }
 
-/* tolower() is locale-specific.  Use a lookup table fast conversion that only
- * operates on ASCII */
-static const unsigned char ares__tolower_lookup[] = {
-  0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C,
-  0x0D, 0x0E, 0x0F, 0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19,
-  0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26,
-  0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33,
-  0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40,
-  0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D,
-  0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A,
-  0x5B, 0x5C, 0x5D, 0x5E, 0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67,
-  0x68, 0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x70, 0x71, 0x72, 0x73, 0x74,
-  0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x80, 0x81,
-  0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E,
-  0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B,
-  0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8,
-  0xA9, 0xAA, 0xAB, 0xAC, 0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5,
-  0xB6, 0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 0xC1, 0xC2,
-  0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF,
-  0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 0xD5, 0xD6, 0xD7, 0xD8, 0xD9, 0xDA, 0xDB, 0xDC,
-  0xDD, 0xDE, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xE4, 0xE5, 0xE6, 0xE7, 0xE8, 0xE9,
-  0xEA, 0xEB, 0xEC, 0xED, 0xEE, 0xEF, 0xF0, 0xF1, 0xF2, 0xF3, 0xF4, 0xF5, 0xF6,
-  0xF7, 0xF8, 0xF9, 0xFA, 0xFB, 0xFC, 0xFD, 0xFE, 0xFF
-};
-
 /* Case insensitive version, meant for ASCII strings */
 unsigned int ares__htable_hash_FNV1a_casecmp(const unsigned char *key,
                                              size_t key_len, unsigned int seed)
@@ -378,7 +408,7 @@ unsigned int ares__htable_hash_FNV1a_casecmp(const unsigned char *key,
   size_t       i;
 
   for (i = 0; i < key_len; i++) {
-    hv ^= (unsigned int)ares__tolower_lookup[key[i]];
+    hv ^= (unsigned int)ares__tolower(key[i]);
     /* hv *= 0x01000193 */
     hv += (hv << 1) + (hv << 4) + (hv << 7) + (hv << 8) + (hv << 24);
   }
