@@ -50,13 +50,6 @@
 #  define INADDR_NONE 0xffffffff
 #endif
 
-#ifdef CARES_EXPOSE_STATICS
-/* Make some internal functions visible for testing */
-#  define STATIC_TESTABLE
-#else
-#  define STATIC_TESTABLE static
-#endif
-
 /* By using a double cast, we can get rid of the bogus warning of
  * warning: cast from 'const struct sockaddr *' to 'const struct sockaddr_in6 *'
  * increases required alignment from 1 to 4 [-Wcast-align]
@@ -123,6 +116,7 @@ typedef struct ares_rand_state ares_rand_state;
 #include "ares__htable_asvp.h"
 #include "ares__buf.h"
 #include "ares_dns_private.h"
+#include "ares__iface_ips.h"
 
 #ifndef HAVE_GETENV
 #  include "ares_getenv.h"
@@ -167,14 +161,17 @@ struct server_connection {
 };
 
 struct server_state {
+  /* Configuration */
   size_t                    idx; /* index for server in system configuration */
-  size_t                    consec_failures; /* Consecutive query failure count
-                                              * can be hard errors or timeouts
-                                              */
   struct ares_addr          addr;
   unsigned short            udp_port;        /* host byte order */
   unsigned short            tcp_port;        /* host byte order */
+  char                      ll_iface[64];    /* IPv6 Link Local Interface */
+  unsigned int              ll_scope;        /* IPv6 Link Local Scope */
 
+  size_t                    consec_failures; /* Consecutive query failure count
+                                              * can be hard errors or timeouts
+                                              */
   ares__llist_t            *connections;
   struct server_connection *tcp_conn;
 
@@ -223,24 +220,9 @@ struct query {
                            * when a query is to be canceled */
 };
 
-/* An IP address pattern; matches an IP address X if X & mask == addr */
-#define PATTERN_MASK 0x1
-#define PATTERN_CIDR 0x2
-
 struct apattern {
-  union {
-    struct in_addr       addr4;
-    struct ares_in6_addr addr6;
-  } addr;
-
-  union {
-    struct in_addr       addr4;
-    struct ares_in6_addr addr6;
-    unsigned short       bits;
-  } mask;
-
-  int            family;
-  unsigned short type;
+  struct ares_addr addr;
+  unsigned char    mask;
 };
 
 struct ares__qcache;
@@ -375,6 +357,9 @@ void ares__rand_bytes(ares_rand_state *state, unsigned char *buf, size_t len);
 
 unsigned short ares__generate_new_id(ares_rand_state *state);
 struct timeval ares__tvnow(void);
+void           ares__timeval_remaining(struct timeval       *remaining,
+                                       const struct timeval *now,
+                                       const struct timeval *tout);
 ares_status_t  ares__expand_name_validated(const unsigned char *encoded,
                                            const unsigned char *abuf,
                                            size_t alen, char **s, size_t *enclen,
@@ -480,9 +465,11 @@ ares_status_t ares__servers_update(ares_channel_t *channel,
 ares_status_t ares__sconfig_append(ares__llist_t         **sconfig,
                                    const struct ares_addr *addr,
                                    unsigned short          udp_port,
-                                   unsigned short          tcp_port);
+                                   unsigned short          tcp_port,
+                                   const char             *ll_iface);
 ares_status_t ares__sconfig_append_fromstr(ares__llist_t **sconfig,
-                                           const char     *str);
+                                           const char     *str,
+                                           ares_bool_t     ignore_invalid);
 ares_status_t ares_in_addr_to_server_config_llist(const struct in_addr *servers,
                                                   size_t          nservers,
                                                   ares__llist_t **llist);
@@ -564,11 +551,17 @@ ares_status_t ares__dns_name_write(ares__buf_t *buf, ares__llist_t **list,
   (x && x->lookups && ares__slist_len(x->servers) > 0 && x->ndots > 0 && \
    x->timeout > 0 && x->tries > 0)
 
+ares_bool_t   ares__subnet_match(const struct ares_addr *addr,
+                                 const struct ares_addr *subnet,
+                                 unsigned char           netmask);
+ares_bool_t   ares__addr_is_linklocal(const struct ares_addr *addr);
+
 size_t        ares__round_up_pow2(size_t n);
 size_t        ares__log2(size_t n);
 size_t        ares__pow(size_t x, size_t y);
 size_t        ares__count_digits(size_t n);
 size_t        ares__count_hexdigits(size_t n);
+unsigned char ares__count_bits_u8(unsigned char x);
 void          ares__qcache_destroy(ares__qcache_t *cache);
 ares_status_t ares__qcache_create(ares_rand_state *rand_state,
                                   unsigned int     max_ttl,
@@ -596,4 +589,12 @@ typedef long long          ares_int64_t;
 typedef unsigned long long ares_uint64_t;
 #endif
 
+#ifdef _WIN32
+#  define HOSTENT_ADDRTYPE_TYPE short
+#  define HOSTENT_LENGTH_TYPE   short
+#else
+#  define HOSTENT_ADDRTYPE_TYPE int
+#  define HOSTENT_LENGTH_TYPE   int
+#endif
+
 #endif /* __ARES_PRIVATE_H */
