@@ -562,6 +562,7 @@ static void process_timeouts(ares_channel_t *channel, struct timeval *now)
     query->timeouts++;
 
     conn = query->conn;
+    server_increment_failures(conn->server);
     ares__requeue_query(query, now);
     ares__check_cleanup_conn(channel, conn);
 
@@ -746,12 +747,14 @@ static void handle_conn_error(struct server_connection *conn,
 {
   struct server_state *server = conn->server;
 
-  /* This will requeue any connections automatically */
-  ares__close_connection(conn);
-
+  /* Increment failures first before requeue so it is unlikely to requeue
+   * to the same server */
   if (critical_failure) {
     server_increment_failures(server);
   }
+
+  /* This will requeue any connections automatically */
+  ares__close_connection(conn);
 }
 
 ares_status_t ares__requeue_query(struct query *query, struct timeval *now)
