@@ -74,8 +74,9 @@ TEST(LibraryInit, BasicChannelInit) {
 }
 
 TEST_F(LibraryTest, OptionsChannelInit) {
-  struct ares_options opts = {0};
+  struct ares_options opts;
   int optmask = 0;
+  memset(&opts, 0, sizeof(opts));
   opts.flags = ARES_FLAG_USEVC | ARES_FLAG_PRIMARY;
   optmask |= ARES_OPT_FLAGS;
   opts.timeout = 2000;
@@ -97,12 +98,12 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   opts.ednspsz = 1280;
   optmask |= ARES_OPT_EDNSPSZ;
   opts.nservers = 2;
-  opts.servers = (struct in_addr *)malloc(opts.nservers * sizeof(struct in_addr));
+  opts.servers = (struct in_addr *)malloc((size_t)opts.nservers * sizeof(struct in_addr));
   opts.servers[0].s_addr = htonl(0x01020304);
   opts.servers[1].s_addr = htonl(0x02030405);
   optmask |= ARES_OPT_SERVERS;
   opts.ndomains = 2;
-  opts.domains = (char **)malloc(opts.ndomains * sizeof(char *));
+  opts.domains = (char **)malloc((size_t)opts.ndomains * sizeof(char *));
   opts.domains[0] = strdup("example.com");
   opts.domains[1] = strdup("example2.com");
   optmask |= ARES_OPT_DOMAINS;
@@ -122,8 +123,9 @@ TEST_F(LibraryTest, OptionsChannelInit) {
   EXPECT_EQ(ARES_SUCCESS, ares_dup(&channel2, channel));
   EXPECT_NE(nullptr, channel2);
 
-  struct ares_options opts2 = {0};
+  struct ares_options opts2;
   int optmask2 = 0;
+  memset(&opts2, 0, sizeof(opts2));
   EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel2, &opts2, &optmask2));
 
   // Note that not all opts-settable fields are saved (e.g.
@@ -167,8 +169,9 @@ TEST_F(LibraryTest, ChannelAllocFail) {
 }
 
 TEST_F(LibraryTest, OptionsChannelAllocFail) {
-  struct ares_options opts = {0};
+  struct ares_options opts;
   int optmask = 0;
+  memset(&opts, 0, sizeof(opts));
   opts.flags = ARES_FLAG_USEVC;
   optmask |= ARES_OPT_FLAGS;
   opts.timeout = 2;
@@ -188,12 +191,12 @@ TEST_F(LibraryTest, OptionsChannelAllocFail) {
   opts.ednspsz = 1280;
   optmask |= ARES_OPT_EDNSPSZ;
   opts.nservers = 2;
-  opts.servers = (struct in_addr *)malloc(opts.nservers * sizeof(struct in_addr));
+  opts.servers = (struct in_addr *)malloc((size_t)opts.nservers * sizeof(struct in_addr));
   opts.servers[0].s_addr = htonl(0x01020304);
   opts.servers[1].s_addr = htonl(0x02030405);
   optmask |= ARES_OPT_SERVERS;
   opts.ndomains = 2;
-  opts.domains = (char **)malloc(opts.ndomains * sizeof(char *));
+  opts.domains = (char **)malloc((size_t)opts.ndomains * sizeof(char *));
   opts.domains[0] = strdup("example.com");
   opts.domains[1] = strdup("example2.com");
   optmask |= ARES_OPT_DOMAINS;
@@ -272,6 +275,25 @@ TEST_F(LibraryTest, EnvInit) {
   ares_destroy(channel);
 }
 
+TEST_F(LibraryTest, EnvInitModernOptions) {
+  ares_channel_t *channel = nullptr;
+  EnvValue v1("LOCALDOMAIN", "this.is.local");
+  EnvValue v2("RES_OPTIONS", "options debug retrans:2 ndots:3 attempts:4 timeout:5 rotate");
+  EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
+
+  channel->optmask |= ARES_OPT_TRIES;
+  channel->optmask |= ARES_OPT_TIMEOUTMS;
+
+  struct ares_options opts;
+  memset(&opts, 0, sizeof(opts));
+  int optmask = 0;
+  EXPECT_EQ(ARES_SUCCESS, ares_save_options(channel, &opts, &optmask));
+  EXPECT_EQ(5000, opts.timeout);
+  EXPECT_EQ(4, opts.tries);
+
+  ares_destroy(channel);
+}
+
 TEST_F(LibraryTest, EnvInitAllocFail) {
   ares_channel_t *channel;
   EnvValue v1("LOCALDOMAIN", "this.is.local");
@@ -304,13 +326,15 @@ TEST_F(DefaultChannelTest, SetSortlistFailures) {
   EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "111.111.111.111/255.255.255.240*"));
   EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "1 0123456789012345"));
   EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "1 /01234567890123456789012345678901"));
-  EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "xyzzy ; lwk"));
-  EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "xyzzy ; 0x123"));
+  EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "xyzzy ; lwk"));
+  EXPECT_EQ(ARES_EBADSTR, ares_set_sortlist(channel_, "xyzzy ; 0x123"));
 }
 
 TEST_F(DefaultChannelTest, SetSortlistVariants) {
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "1.2.3.4"));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "1.2.3.4 ; 2.3.4.5"));
+  EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "1.2.3.4/26;1234::5678/126;4.5.6.7;5678::1234"));
+  EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, " 1.2.3.4/26 1234::5678/126   4.5.6.7 5678::1234  "));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "129.1.1.1"));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "192.1.1.1"));
   EXPECT_EQ(ARES_SUCCESS, ares_set_sortlist(channel_, "224.1.1.1"));
@@ -369,8 +393,8 @@ CONTAINED_TEST_F(LibraryTest, ContainerChannelInit,
                  "myhostname", "mydomainname.org", filelist) {
   ares_channel_t *channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
-  std::vector<std::string> actual = GetNameServers(channel);
-  std::vector<std::string> expected = {"1.2.3.4:53"};
+  std::string actual = GetNameServers(channel);
+  std::string expected = "1.2.3.4:53";
   EXPECT_EQ(expected, actual);
   EXPECT_EQ(2, channel->ndomains);
   EXPECT_EQ(std::string("first.com"), std::string(channel->domains[0]));
@@ -618,11 +642,9 @@ CONTAINED_TEST_F(LibraryTest, ContainerBlacklistedIpv6,
                  "myhostname", "mydomainname.org", blacklistedIpv6) {
   ares_channel_t *channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
-  std::vector<std::string> actual = GetNameServers(channel);
-  std::vector<std::string> expected = {
-    "254.192.1.1:53",
-    "[ffc0:0000:0000:0000:0000:0000:0000:c001]:53"
-  };
+  std::string actual = GetNameServers(channel);
+  std::string expected = "254.192.1.1:53,"
+                         "[ffc0::c001]:53";
   EXPECT_EQ(expected, actual);
 
   EXPECT_EQ(1, channel->ndomains);
@@ -640,8 +662,8 @@ CONTAINED_TEST_F(LibraryTest, ContainerMultiResolvInit,
                  "myhostname", "mydomainname.org", multiresolv) {
   ares_channel_t *channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
-  std::vector<std::string> actual = GetNameServers(channel);
-  std::vector<std::string> expected = {"[0001:0000:0000:0000:0000:0000:0000:0002]:53"};
+  std::string actual = GetNameServers(channel);
+  std::string expected = "[1::2]:53";
   EXPECT_EQ(expected, actual);
 
   EXPECT_EQ(1, channel->ndomains);
@@ -671,8 +693,8 @@ CONTAINED_TEST_F(LibraryTest, ContainerEmptyInit,
                  "host.domain.org", "domain.org", empty) {
   ares_channel_t *channel = nullptr;
   EXPECT_EQ(ARES_SUCCESS, ares_init(&channel));
-  std::vector<std::string> actual = GetNameServers(channel);
-  std::vector<std::string> expected = {"127.0.0.1:53"};
+  std::string actual = GetNameServers(channel);
+  std::string expected = "127.0.0.1:53";
   EXPECT_EQ(expected, actual);
 
   EXPECT_EQ(1, channel->ndomains);
