@@ -1,17 +1,25 @@
-/*
- * Copyright (C) Brad House
+/* MIT License
  *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of M.I.T. not be used in
- * advertising or publicity pertaining to distribution of the
- * software without specific, written prior permission.
- * M.I.T. makes no representations about the suitability of
- * this software for any purpose.  It is provided "as is"
- * without express or implied warranty.
+ * Copyright (c) The c-ares project and its contributors
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to deal
+ * in the Software without restriction, including without limitation the rights
+ * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+ * copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
+ * SOFTWARE.
  *
  * SPDX-License-Identifier: MIT
  */
@@ -21,10 +29,23 @@
 #include "ares-test.h"
 #include "ares-test-ai.h"
 #include "dns-proto.h"
-
-// Include ares internal files for DNS protocol details
 #include "ares_dns.h"
 
+extern "C" {
+// Remove command-line defines of package variables for the test project...
+#undef PACKAGE_NAME
+#undef PACKAGE_BUGREPORT
+#undef PACKAGE_STRING
+#undef PACKAGE_TARNAME
+// ... so we can include the library's config without symbol redefinitions.
+#include "ares_setup.h"
+#include "ares_inet_net_pton.h"
+#include "ares_data.h"
+#include "ares_strsplit.h"
+#include "ares_private.h"
+}
+
+
 #ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
@@ -49,8 +70,8 @@ namespace ares {
 namespace test {
 
 bool verbose = false;
-static constexpr int dynamic_port = 0;
-int mock_port = dynamic_port;
+static constexpr unsigned short dynamic_port = 0;
+unsigned short mock_port = dynamic_port;
 
 const std::vector<int> both_families = {AF_INET, AF_INET6};
 const std::vector<int> ipv4_family = {AF_INET};
@@ -79,12 +100,36 @@ unsigned long long LibraryTest::fails_ = 0;
 std::map<size_t, int> LibraryTest::size_fails_;
 
 void ProcessWork(ares_channel_t *channel,
-                 std::function<std::set<int>()> get_extrafds,
-                 std::function<void(int)> process_extra) {
+                 std::function<std::set<ares_socket_t>()> get_extrafds,
+                 std::function<void(ares_socket_t)> process_extra,
+                 unsigned int cancel_ms) {
   int nfds, count;
   fd_set readers, writers;
-  struct timeval tv;
+
+#ifndef CARES_SYMBOL_HIDING
+  struct timeval tv_begin  = ares__tvnow();
+  struct timeval tv_cancel = tv_begin;
+
+  if (cancel_ms) {
+    if (verbose) std::cerr << "ares_cancel will be called after " << cancel_ms << "ms" << std::endl;
+    tv_cancel.tv_sec  += (cancel_ms / 1000);
+    tv_cancel.tv_usec += ((cancel_ms % 1000) * 1000);
+  }
+#else
+  if (cancel_ms) {
+    std::cerr << "library built with symbol hiding, can't test with cancel support" << std::endl;
+    return;
+  }
+#endif
+
   while (true) {
+#ifndef CARES_SYMBOL_HIDING
+    struct timeval  tv_now = ares__tvnow();
+    struct timeval  tv_remaining;
+#endif
+    struct timeval  tv;
+    struct timeval *tv_select;
+
     // Retrieve the set of file descriptors that the library wants us to monitor.
     FD_ZERO(&readers);
     FD_ZERO(&writers);
@@ -93,20 +138,39 @@ void ProcessWork(ares_channel_t *channel,
       return;
 
     // Add in the extra FDs if present.
-    std::set<int> extrafds = get_extrafds();
-    for (int extrafd : extrafds) {
+    std::set<ares_socket_t> extrafds = get_extrafds();
+    for (ares_socket_t extrafd : extrafds) {
       FD_SET(extrafd, &readers);
-      if (extrafd >= nfds) {
-        nfds = extrafd + 1;
+      if (extrafd >= (ares_socket_t)nfds) {
+        nfds = (int)extrafd + 1;
       }
     }
 
     /* If ares_timeout returns NULL, it means there are no requests in queue,
      * so we can break out */
-    if (ares_timeout(channel, NULL, &tv) == NULL)
+    tv_select = ares_timeout(channel, NULL, &tv);
+    if (tv_select == NULL)
       return;
 
-    count = select(nfds, &readers, &writers, nullptr, &tv);
+#ifndef CARES_SYMBOL_HIDING
+    if (cancel_ms) {
+      unsigned int remaining_ms;
+      ares__timeval_remaining(&tv_remaining,
+                              &tv_now,
+                              &tv_cancel);
+      remaining_ms = (unsigned int)((tv_remaining.tv_sec * 1000) + (tv_remaining.tv_usec / 1000));
+      if (remaining_ms == 0) {
+        if (verbose) std::cerr << "Issuing ares_cancel()" << std::endl;
+        ares_cancel(channel);
+        cancel_ms = 0; /* Disable issuing cancel again */
+      } else {
+        /* Recalculate proper timeout since we also have a cancel to wait on */
+        tv_select = ares_timeout(channel, &tv_remaining, &tv);
+      }
+    }
+#endif
+
+    count = select(nfds, &readers, &writers, nullptr, tv_select);
     if (count < 0) {
       fprintf(stderr, "select() failed, errno %d\n", errno);
       return;
@@ -116,7 +180,7 @@ void ProcessWork(ares_channel_t *channel,
     ares_process(channel, &readers, &writers);
 
     // Let the provided callback process any activity on the extra FD.
-    for (int extrafd : extrafds) {
+    for (ares_socket_t extrafd : extrafds) {
       if (FD_ISSET(extrafd, &readers)) {
         process_extra(extrafd);
       }
@@ -179,29 +243,29 @@ void LibraryTest::afree(void *ptr) {
   free(ptr);
 }
 
-std::set<int> NoExtraFDs() {
-  return std::set<int>();
+std::set<ares_socket_t> NoExtraFDs() {
+  return std::set<ares_socket_t>();
 }
 
-void DefaultChannelTest::Process() {
-  ProcessWork(channel_, NoExtraFDs, nullptr);
+void DefaultChannelTest::Process(unsigned int cancel_ms) {
+  ProcessWork(channel_, NoExtraFDs, nullptr, cancel_ms);
 }
 
-void FileChannelTest::Process() {
-  ProcessWork(channel_, NoExtraFDs, nullptr);
+void FileChannelTest::Process(unsigned int cancel_ms) {
+  ProcessWork(channel_, NoExtraFDs, nullptr, cancel_ms);
 }
 
-void DefaultChannelModeTest::Process() {
-  ProcessWork(channel_, NoExtraFDs, nullptr);
+void DefaultChannelModeTest::Process(unsigned int cancel_ms) {
+  ProcessWork(channel_, NoExtraFDs, nullptr, cancel_ms);
 }
 
-MockServer::MockServer(int family, int port)
+MockServer::MockServer(int family, unsigned short port)
   : udpport_(port), tcpport_(port), qid_(-1) {
   // Create a TCP socket to receive data on.
   tcp_data_ = NULL;
   tcp_data_len_ = 0;
   tcpfd_ = socket(family, SOCK_STREAM, 0);
-  EXPECT_NE(-1, tcpfd_);
+  EXPECT_NE(ARES_SOCKET_BAD, tcpfd_);
   int optval = 1;
   setsockopt(tcpfd_, SOL_SOCKET, SO_REUSEADDR,
              BYTE_CAST &optval , sizeof(int));
@@ -211,7 +275,7 @@ MockServer::MockServer(int family, int port)
 
   // Create a UDP socket to receive data on.
   udpfd_ = socket(family, SOCK_DGRAM, 0);
-  EXPECT_NE(-1, udpfd_);
+  EXPECT_NE(ARES_SOCKET_BAD, udpfd_);
 
   // Bind the sockets to the given port.
   if (family == AF_INET) {
@@ -280,7 +344,7 @@ MockServer::MockServer(int family, int port)
 }
 
 MockServer::~MockServer() {
-  for (int fd : connfds_) {
+  for (ares_socket_t fd : connfds_) {
     sclose(fd);
   }
   sclose(tcpfd_);
@@ -288,7 +352,7 @@ MockServer::~MockServer() {
   free(tcp_data_);
 }
 
-void MockServer::ProcessPacket(int fd, struct sockaddr_storage *addr, socklen_t addrlen,
+void MockServer::ProcessPacket(ares_socket_t fd, struct sockaddr_storage *addr, ares_socklen_t addrlen,
                                byte *data, int len) {
 
   // Assume the packet is a well-formed DNS request and extract the request
@@ -312,8 +376,8 @@ void MockServer::ProcessPacket(int fd, struct sockaddr_storage *addr, socklen_t
               << ")" << std::endl;
     return;
   }
-  byte* question = data + 12;
-  int qlen = len - 12;
+  byte* question = data + NS_HFIXEDSZ;
+  int qlen = len - NS_HFIXEDSZ;
 
   char *name = nullptr;
   long enclen;
@@ -322,7 +386,11 @@ void MockServer::ProcessPacket(int fd, struct sockaddr_storage *addr, socklen_t
     std::cerr << "Failed to retrieve name" << std::endl;
     return;
   }
-  qlen -= enclen;
+  if (enclen > qlen) {
+    std::cerr << "(error, encoded name len " << enclen << "bigger than remaining data " << qlen << " bytes)" << std::endl;
+    return;
+  }
+  qlen -= (int)enclen;
   question += enclen;
   std::string namestr(name);
   ares_free_string(name);
@@ -350,13 +418,13 @@ void MockServer::ProcessPacket(int fd, struct sockaddr_storage *addr, socklen_t
 
 }
 
-void MockServer::ProcessFD(int fd) {
+void MockServer::ProcessFD(ares_socket_t fd) {
   if (fd != tcpfd_ && fd != udpfd_ && connfds_.find(fd) == connfds_.end()) {
     // Not one of our FDs.
     return;
   }
   if (fd == tcpfd_) {
-    int connfd = accept(tcpfd_, NULL, NULL);
+    ares_socket_t connfd = accept(tcpfd_, NULL, NULL);
     if (connfd < 0) {
       std::cerr << "Error accepting connection on fd " << fd << std::endl;
     } else {
@@ -369,11 +437,11 @@ void MockServer::ProcessFD(int fd) {
   struct sockaddr_storage addr;
   socklen_t addrlen = sizeof(addr);
   byte buffer[2048];
-  int len = recvfrom(fd, BYTE_CAST buffer, sizeof(buffer), 0,
+  ares_ssize_t len = (ares_ssize_t)recvfrom(fd, BYTE_CAST buffer, sizeof(buffer), 0,
                      (struct sockaddr *)&addr, &addrlen);
 
   if (fd != udpfd_) {
-    if (len == 0) {
+    if (len <= 0) {
       connfds_.erase(std::find(connfds_.begin(), connfds_.end(), fd));
       sclose(fd);
       free(tcp_data_);
@@ -381,18 +449,18 @@ void MockServer::ProcessFD(int fd) {
       tcp_data_len_ = 0;
       return;
     }
-    tcp_data_ = (unsigned char *)realloc(tcp_data_, tcp_data_len_ + len);
-    memcpy(tcp_data_ + tcp_data_len_, buffer, len);
-    tcp_data_len_ += len;
+    tcp_data_ = (unsigned char *)realloc(tcp_data_, tcp_data_len_ + (size_t)len);
+    memcpy(tcp_data_ + tcp_data_len_, buffer, (size_t)len);
+    tcp_data_len_ += (size_t)len;
 
     /* TCP might aggregate the various requests into a single packet, so we
      * need to split */
     while (tcp_data_len_ > 2) {
-      size_t tcplen = (tcp_data_[0] << 8) + tcp_data_[1];
+      size_t tcplen = ((size_t)tcp_data_[0] << 8) + (size_t)tcp_data_[1];
       if (tcp_data_len_ - 2 < tcplen)
         break;
 
-      ProcessPacket(fd, &addr, addrlen, tcp_data_ + 2, tcplen);
+      ProcessPacket(fd, &addr, addrlen, tcp_data_ + 2, (int)tcplen);
 
       /* strip off processed data if connection not terminated */
       if (tcp_data_ != NULL) {
@@ -402,19 +470,19 @@ void MockServer::ProcessFD(int fd) {
     }
   } else {
     /* UDP is always a single packet */
-    ProcessPacket(fd, &addr, addrlen, buffer, len);
+    ProcessPacket(fd, &addr, addrlen, buffer, (int)len);
   }
 
 }
 
-std::set<int> MockServer::fds() const {
-  std::set<int> result = connfds_;
+std::set<ares_socket_t> MockServer::fds() const {
+  std::set<ares_socket_t> result = connfds_;
   result.insert(tcpfd_);
   result.insert(udpfd_);
   return result;
 }
 
-void MockServer::ProcessRequest(int fd, struct sockaddr_storage* addr, int addrlen,
+void MockServer::ProcessRequest(ares_socket_t fd, struct sockaddr_storage* addr, ares_socklen_t addrlen,
                                 int qid, const std::string& name, int rrtype) {
   // Before processing, let gMock know the request is happening.
   OnRequest(name, rrtype);
@@ -440,7 +508,7 @@ void MockServer::ProcessRequest(int fd, struct sockaddr_storage* addr, int addrl
 
   // Prefix with 2-byte length if TCP.
   if (fd != udpfd_) {
-    int len = reply.size();
+    int len = (int)reply.size();
     std::vector<byte> vlen = {(byte)((len & 0xFF00) >> 8), (byte)(len & 0xFF)};
     reply.insert(reply.begin(), vlen.begin(), vlen.end());
     // Also, don't bother with the destination address.
@@ -448,19 +516,19 @@ void MockServer::ProcessRequest(int fd, struct sockaddr_storage* addr, int addrl
     addrlen = 0;
   }
 
-  int rc = sendto(fd, BYTE_CAST reply.data(), reply.size(), 0,
+  ares_ssize_t rc = (ares_ssize_t)sendto(fd, BYTE_CAST reply.data(), (SEND_TYPE_ARG3)reply.size(), 0,
                   (struct sockaddr *)addr, addrlen);
-  if (rc < static_cast<int>(reply.size())) {
+  if (rc < static_cast<ares_ssize_t>(reply.size())) {
     std::cerr << "Failed to send full reply, rc=" << rc << std::endl;
   }
 }
 
 // static
-MockChannelOptsTest::NiceMockServers MockChannelOptsTest::BuildServers(int count, int family, int base_port) {
+MockChannelOptsTest::NiceMockServers MockChannelOptsTest::BuildServers(int count, int family, unsigned short base_port) {
   NiceMockServers servers;
   assert(count > 0);
-  for (int ii = 0; ii < count; ii++) {
-    int port = base_port == dynamic_port ? dynamic_port : base_port + ii;
+  for (unsigned short ii = 0; ii < count; ii++) {
+    unsigned short port = base_port == dynamic_port ? dynamic_port : base_port + ii;
     std::unique_ptr<NiceMockServer> server(new NiceMockServer(family, port));
     servers.push_back(std::move(server));
   }
@@ -544,10 +612,7 @@ MockChannelOptsTest::MockChannelOptsTest(int count,
   }
   if (verbose) {
     std::cerr << "Configured library with servers:";
-    std::vector<std::string> servers = GetNameServers(channel_);
-    for (const auto& server : servers) {
-      std::cerr << " " << server;
-    }
+    std::cerr << GetNameServers(channel_);
     std::cerr << std::endl;
   }
 }
@@ -559,26 +624,27 @@ MockChannelOptsTest::~MockChannelOptsTest() {
   channel_ = nullptr;
 }
 
-std::set<int> MockChannelOptsTest::fds() const {
-  std::set<int> fds;
+std::set<ares_socket_t> MockChannelOptsTest::fds() const {
+  std::set<ares_socket_t> fds;
   for (const auto& server : servers_) {
-    std::set<int> serverfds = server->fds();
+    std::set<ares_socket_t> serverfds = server->fds();
     fds.insert(serverfds.begin(), serverfds.end());
   }
   return fds;
 }
 
-void MockChannelOptsTest::ProcessFD(int fd) {
+void MockChannelOptsTest::ProcessFD(ares_socket_t fd) {
   for (auto& server : servers_) {
     server->ProcessFD(fd);
   }
 }
 
-void MockChannelOptsTest::Process() {
+void MockChannelOptsTest::Process(unsigned int cancel_ms) {
   using namespace std::placeholders;
   ProcessWork(channel_,
               std::bind(&MockChannelOptsTest::fds, this),
-              std::bind(&MockChannelOptsTest::ProcessFD, this, _1));
+              std::bind(&MockChannelOptsTest::ProcessFD, this, _1),
+              cancel_ms);
 }
 
 std::ostream& operator<<(std::ostream& os, const HostResult& result) {
@@ -700,12 +766,12 @@ std::ostream& operator<<(std::ostream& os, const AddrInfo& ai) {
     unsigned short port = 0;
     os << "addr=[";
     if(next->ai_family == AF_INET) {
-      sockaddr_in* sin = (sockaddr_in*)next->ai_addr;
+      sockaddr_in* sin = (sockaddr_in *)((void *)next->ai_addr);
       port = ntohs(sin->sin_port);
       os << AddressToString(&sin->sin_addr, 4);
     }
     else if (next->ai_family == AF_INET6) {
-      sockaddr_in6* sin = (sockaddr_in6*)next->ai_addr;
+      sockaddr_in6* sin = (sockaddr_in6*)((void *)next->ai_addr);
       port = ntohs(sin->sin6_port);
       os << "[" << AddressToString(&sin->sin6_addr, 16) << "]";
     }
@@ -779,38 +845,14 @@ void NameInfoCallback(void *data, int status, int timeouts,
   if (verbose) std::cerr << "NameInfoCallback(" << *result << ")" << std::endl;
 }
 
-std::vector<std::string> GetNameServers(ares_channel_t *channel) {
-  struct ares_addr_port_node* servers = nullptr;
-  EXPECT_EQ(ARES_SUCCESS, ares_get_servers_ports(channel, &servers));
-  struct ares_addr_port_node* server = servers;
-  std::vector<std::string> results;
-  while (server) {
-    std::stringstream ss;
-    switch (server->family) {
-    case AF_INET:
-      ss << AddressToString((char*)&server->addr.addr4, 4);
-      break;
-    case AF_INET6:
-      if (server->udp_port != 0) {
-        ss << '[';
-      }
-      ss << AddressToString((char*)&server->addr.addr6, 16);
-      if (server->udp_port != 0) {
-        ss << ']';
-      }
-      break;
-    default:
-      results.push_back("<unknown family>");
-      break;
-    }
-    if (server->udp_port != 0) {
-      ss << ":" << server->udp_port;
-    }
-    results.push_back(ss.str());
-    server = server->next;
-  }
-  if (servers) ares_free_data(servers);
-  return results;
+std::string GetNameServers(ares_channel_t *channel) {
+  char *csv = ares_get_servers_csv(channel);
+  EXPECT_NE((char *)NULL, csv);
+
+  std::string servers(csv);
+
+  ares_free_string(csv);
+  return servers;
 }
 
 TransientDir::TransientDir(const std::string& dirname) : dirname_(dirname) {
@@ -831,8 +873,8 @@ TransientFile::TransientFile(const std::string& filename,
     std::cerr << "Error: failed to create '" << filename << "'" << std::endl;
     return;
   }
-  int rc = fwrite(contents.data(), 1, contents.size(), f);
-  if (rc != (int)contents.size()) {
+  size_t rc = (size_t)fwrite(contents.data(), 1, contents.size(), f);
+  if (rc != contents.size()) {
     std::cerr << "Error: failed to write contents of '" << filename << "'" << std::endl;
   }
   fclose(f);
