@@ -20,7 +20,9 @@
 #define ARES_TEST_H
 
 #include "ares_setup.h"
-#include "ares.h"
+#ifdef HAVE_CONFIG_H
+#include "ares_config.h"
+#endif
 
 #include "dns-proto.h"
 // Include ares internal file for DNS protocol constants
@@ -29,9 +31,6 @@
 #include "gtest/gtest.h"
 #include "gmock/gmock.h"
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
 #if defined(HAVE_USER_NAMESPACE) && defined(HAVE_UTS_NAMESPACE)
 #  define HAVE_CONTAINER
 #endif
@@ -69,7 +68,8 @@ extern std::vector<std::pair<int, bool>>       families_modes;
 // optionally the given set-of-FDs + work function.
 void                    ProcessWork(ares_channel_t                          *channel,
                                     std::function<std::set<ares_socket_t>()> get_extrafds,
-                                    std::function<void(ares_socket_t)>       process_extra);
+                                    std::function<void(ares_socket_t)>       process_extra,
+                                    unsigned int                             cancel_ms = 0);
 std::set<ares_socket_t> NoExtraFDs();
 
 // Test fixture that ensures library initialization, and allows
@@ -128,7 +128,7 @@ public:
   }
 
   // Process all pending work on ares-owned file descriptors.
-  void Process();
+  void Process(unsigned int cancel_ms = 0);
 
 protected:
   ares_channel_t *channel_;
@@ -155,7 +155,7 @@ public:
   }
 
   // Process all pending work on ares-owned file descriptors.
-  void Process();
+  void Process(unsigned int cancel_ms = 0);
 
 protected:
   ares_channel_t *channel_;
@@ -184,7 +184,7 @@ public:
   }
 
   // Process all pending work on ares-owned file descriptors.
-  void Process();
+  void Process(unsigned int cancel_ms = 0);
 
 protected:
   ares_channel_t *channel_;
@@ -271,7 +271,7 @@ public:
 
   // Process all pending work on ares-owned and mock-server-owned file
   // descriptors.
-  void Process();
+  void Process(unsigned int cancel_ms = 0);
 
 protected:
   // NiceMockServer doesn't complain about uninteresting calls.
@@ -339,7 +339,7 @@ ACTION_P2(CancelChannel, mockserver, channel)
 }
 
 // gMock action to disconnect all connections.
-ACTION_P2(Disconnect, mockserver)
+ACTION_P(Disconnect, mockserver)
 {
   mockserver->Disconnect();
 }
@@ -444,7 +444,7 @@ void AddrInfoCallback(void *data, int status, int timeouts,
                       struct ares_addrinfo *res);
 
 // Retrieve the name servers used by a channel.
-std::vector<std::string> GetNameServers(ares_channel_t *channel);
+std::string GetNameServers(ares_channel_t *channel);
 
 // RAII class to temporarily create a directory of a given name.
 class TransientDir {
